@@ -43,12 +43,8 @@ sub new {
      return $CGI::Q = $self->SUPER::new($initializer, @param);
 }
 
-package FCGI::Stream;
-
 # to work around an IPC::Run issue...
-sub FILENO {
-    1;
-}
+*FCGI::Stream::FILENO = sub { 1; };
 
 1;
 
