@@ -82,7 +82,7 @@ sub GET_access_rule {
     my $sql;
     if ($value eq '~') {
         $sql = [:sql|
-            select id, method, prefix, segments, prohibiting
+            select id, method, prefix, segments, array_to_string(applied_to,' ') as applied_to, prohibiting
             from _access
             where role = $role;
         |];
@@ -96,14 +96,14 @@ sub GET_access_rule {
 
         if ($col eq '~') {
             $sql = [:sql|
-                select id, method, prefix, segments, prohibiting
+                select id, method, prefix, segments, array_to_string(applied_to,' ') as applied_to, prohibiting
                 from _access where role = $role and
                     (id::text $kw:op $value or method $kw:op $value or
                      prefix $kw:op $value);
             |];
         } else {
             $sql = [:sql|
-                select id, method, prefix, segments, prohibiting
+                select id, method, prefix, segments, array_to_string(applied_to,' ') as applied_to, prohibiting
                 from _access
                 where role = $role and $sym:col $kw:op $value;
             |];
@@ -148,7 +148,8 @@ sub PUT_access_rule {
     if ($role eq 'Admin') {
         die "Role \"Admin\" is read only.\n";
     }
-    if ($col ne '~' and $col ne 'method' and $col ne 'url' and $col ne 'id') {
+    if ($col ne '~' and $col ne 'method' and $col ne 'url' and
+                        $col ne 'applied_to' and $col ne 'id') {
         die "Unknown access rule field: $col\n";
     }
 
@@ -168,6 +169,12 @@ sub PUT_access_rule {
             my $segs = @bits;
             $update->set(prefix => Q($prefix));
             $update->set(segments => Q($segs));
+        } elsif ($col eq 'applied_to') {
+            if ($val !~ /^[\d|\.|\/|\s]+$/) {
+                die "the acl column applied_to: $applied_to's format must be cidr type seperated by space.";
+            }
+            $val =~ s/\s+/,/g;
+            $update->set(applied_to => Q("{$val}"));
         } else {
             $update->set(QI($col) => Q($val));
         }
@@ -251,6 +258,12 @@ sub insert_rule {
     if (!defined $url) {
         die "row $row: Column \"url\" is missing.\n";
     }
+    my $applied_to = delete $data->{applied_to} || '0.0.0.0/0';
+    if ($applied_to !~ /^[\d|\.|\/|\s]+$/) {
+        die "the acl column applied_to: $applied_to's format must be cidr type seperated by space.";
+    }
+    $applied_to =~ s/\s+/,/g;
+    $applied_to = "{$applied_to}";
     my $prohibiting = delete $data->{prohibiting} ? 'true' : 'false';
     if ($url !~ s/^\/=\/+//) {
         die "URL must be lead by \"/=/\".\n";
@@ -265,8 +278,8 @@ sub insert_rule {
     $prefix = '' if $prefix eq '~';
     my $segs = @bits;
     my $sql = [:sql|
-        insert into _access (role, method, prefix, segments, prohibiting)
-        values ($role, $method, $prefix, $segs, $prohibiting) |];
+        insert into _access (role, method, prefix, segments, applied_to, prohibiting)
+        values ($role, $method, $prefix, $segs, $applied_to, $prohibiting) |];
     return $openresty->do($sql);
 }
 
@@ -302,6 +315,7 @@ sub GET_role {
     $res->{columns} = [
         { name => "method", type => "text", label => "HTTP method" },
         { name => "url", type => "text", label => "Resource"},
+        { name => "applied_to", type => "text", label => "Applied_to"},
         { name => "prohibiting", type => "boolean", label => "Prohibiting"}
     ];
     return $res;
@@ -355,12 +369,14 @@ sub role_count {
 
 sub new_role {
     my ($self, $openresty, $data) = @_;
-    my $nroles = $self->role_count($openresty);
-    my $res;
-    if ($nroles >= $ROLE_LIMIT) {
-        die "Exceeded role count limit $ROLE_LIMIT.\n";
-    }
 
+    if (!$openresty->is_unlimited) {
+        my $nroles = $self->role_count($openresty);
+        if ($nroles >= $ROLE_LIMIT) {
+            die "Exceeded role count limit $ROLE_LIMIT.\n";
+        }
+    }
+    my $res;
     my $name = delete $data->{name} or
         die "No 'name' specified.\n";
     _IDENT($name) or die "Bad role name: ", $OpenResty::Dumper->($name), "\n";
@@ -380,18 +396,18 @@ sub new_role {
     }
     _STRING($login) or die "Bad 'login' value: ", $OpenResty::Dumper->($login), "\n";
 
-    if ($login !~ /^(?:password|captcha|anonymous)$/) {
+    if ($login !~ /^(?:captcha\+password|password|captcha|anonymous)$/) {
         die "Unknown login method: $login\n";
     }
 
     my $password = delete $data->{password};
-    if (defined $password and $login ne 'password') {
+    if (defined $password and $login !~ /password$/) {
         $openresty->warning("Field 'password' ignored.");
     }
 
-    if ($login eq 'password') {
+    if ($login eq 'password' or $login eq 'captcha+password') {
         if (!defined $password) {
-            die "No password given when 'login' is 'password'.\n";
+            die "No password given when 'login' is '$login'.\n";
         } elsif (length($password) < $PASSWORD_MIN_LEN) {
             die "Password too short; at least $PASSWORD_MIN_LEN chars required.\n";
         }
@@ -433,7 +449,7 @@ sub PUT_role {
     if (defined $new_login) {
         _STRING($new_login) or
             die "Bad login method: ", $OpenResty::Dumper->($new_login), "\n";
-        if ($new_login !~ /^(?:password|anonymous|captcha)$/) {
+        if ($new_login !~ /^(?:captcha\+password|password|anonymous|captcha)$/) {
             die "Bad login method: $new_login\n";
         }
         $OpenResty::Cache->remove_has_role($user, $role);
@@ -447,12 +463,17 @@ sub PUT_role {
         }
         _STRING($new_password) or
             die "Bad password: ", $OpenResty::Dumper->($new_password), "\n";
+        if (length($new_password) < $PASSWORD_MIN_LEN) {
+            die "Password too short; at least $PASSWORD_MIN_LEN chars required.\n";
+        }
+
         #check_password($new_password);
         $update->set(password => Q($new_password));
     }
 
-    if (defined $new_login and $new_login eq 'password' and !defined $new_password) {
-        die "No password given when 'login' is 'password'.\n";
+    if (defined $new_login and !defined $new_password and
+            ($new_login eq 'password' or $new_login eq 'captcha+password')) {
+        die "No password given when 'login' is '$new_login'.\n";
     }
 
     my $new_desc = delete $data->{description};
@@ -471,6 +492,7 @@ sub PUT_role {
 sub current_user_can {
     my ($self, $openresty, $meth, $bits) = @_;
     my $role = $openresty->{_role};
+    my $client_ip = $openresty->{_client_ip};
     return 1 if $role eq 'Admin'; # short cut
     my $url = join '/', @$bits;
     my $segs = @$bits;
@@ -478,10 +500,14 @@ sub current_user_can {
         select prohibiting
         from _access
         where role = $role and method = $meth and segments = $segs
-            and $url like (prefix || '%')
-        order by prohibiting desc
-        limit 1;
-    |];
+            and $url like (prefix || '%') |];
+    if (!$client_ip) {
+        die "Cannot get client ip address.\n";
+    }
+    if ( $client_ip ne '127.0.0.1') {
+        $sql .= [:sql| and cidr($client_ip) <<= any(applied_to) |];
+    }
+    $sql .= " order by prohibiting desc limit 1";
     ### $sql
     my $res = $openresty->select($sql);
     if ($res && @$res) {
